import { VFS } from './VFS';
import { SourceNode } from '@journeyapps/source-utils';
import { SourceEntry } from './SourceEntry';

export interface StaticEntry {
  content: string;

  path: string;

  /**
   * True if the file should be loaded as a script (entry point).
   *
   * It is generally not required to add libraries / imported files, since they are resolved automatically.
   */
  isScript: boolean;

  /**
   * True if the file can be imported.
   */
  isModule: boolean;

  generated: boolean;
}

/**
 * Serves static files - typically generated, but could also be used for bundled files.
 */
export class StaticVFS implements VFS {
  private entries: Record<string, SourceEntry> = {};
  private nodes: Record<string, SourceNode> = {};

  constructor(entries?: StaticEntry[]) {
    if (entries) {
      this.add(...entries);
    }
  }

  add(...entries: StaticEntry[]) {
    for (let entry of entries) {
      this.entries[entry.path] = {
        path: entry.path,
        isModule: entry.isModule,
        isScript: entry.isScript,
        generated: entry.generated,
        version: '1'
      };
      this.nodes[entry.path] = SourceNode.createDirectNode({ content: entry.content, sourceFile: entry.path });
    }
  }

  getTargetsFor(source: string): string[] | null {
    // Even if the file is generated, it does not have sources.
    return null;
  }

  info(path: string): SourceEntry | null {
    return this.entries[path] ?? null;
  }

  listFiles(): string[] {
    return Object.keys(this.entries);
  }

  read(path: string): SourceNode | null {
    return this.nodes[path] ?? null;
  }
}
